/***************************************************************************
                          molecule_13nmr.cpp  -  13C NMR tools for Molecules
                             -------------------
    begin                : Thu Jul 25 2002
    copyright            : (C) 2002 by Adam Tenderholt
    email                : atenderholt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
//#include <qobject.h>
#include <qlist.h>
#include <qstring.h>
//#include <qmessagebox.h>
#include <iostream.h>
#include <math.h>
#include <klocale.h>
#include "graphdialog.h"
//#include "molinfodialog.h"
//#include "moldata.h"
//#include "render2d.h"
//#include "drawable.h"
#include "molecule.h"
#include "dpoint.h"
//#include "defs.h"
//#include "namer.h"

// Prediction stuff
#include "boshcp.h"
// SSSR (see comments in this file)
#include "molecule_sssr.h"

QStringList Molecule::Calc13CNMR(bool show_dialog) {
  BremserOneSphereHOSECodePredictor boshcp;
  QStringList hosecodes;
  QList<DPoint> up;
  QList<DPoint> sphere1;
  QString tmp_str, ts;
  DPoint *tmp_pt2, *tmp_pt3;
  int hs, hsmax;

  // get list of unique points
  up = AllPoints();
  // find rings (esp. find aromaticity) - do after CopyTextToDPoint()
  MakeSSSR();

  // Scan for keto groups
  for (tmp_pt = up.first(); tmp_pt != NULL; tmp_pt = up.next()) {
    tmp_pt->ketos = 0;
    for (tmp_bond = bonds.first(); tmp_bond != 0; tmp_bond = bonds.next()) {
      if (tmp_bond->Find(tmp_pt) == true) {
	tmp_pt2 = tmp_bond->otherPoint(tmp_pt);
	if ( (tmp_pt2->element == "O") && (tmp_bond->Order() == 2) )
	  tmp_pt->ketos += 1;
      }
    }
  }
  // Determine atoms surrounding each atom and build HOSE code list
  for (tmp_pt = up.first(); tmp_pt != NULL; tmp_pt = up.next()) {
    if (tmp_pt->element == "C") {  // only look at carbons
      for (tmp_bond = bonds.first(); tmp_bond != 0; tmp_bond = bonds.next()) {
	if (tmp_bond->Find(tmp_pt) == true) {
	  tmp_str = "";
	  hs = 0;
	  tmp_pt2 = tmp_bond->otherPoint(tmp_pt);
	  if (tmp_bond->Order() == 3) { tmp_str.append("%"); hs += 300; }
	  if ( (tmp_pt->aromatic == true) && (tmp_pt2->aromatic == true) ) {
	    tmp_str.append("*"); hs += 100;
	  } else {
	    if (tmp_bond->Order() == 2) {
	      if (tmp_bond->Dash() == 0)
		{ tmp_str.append("="); hs += 200; }
	      if (tmp_bond->Dash() == 1)
		{ tmp_str.append("*"); hs += 100; }
	    }
	  }
	  if (tmp_pt2->element == "C") { tmp_str.append("C"); hs += 24; }
	  if (tmp_pt2->element == "O") { tmp_str.append("O"); hs += 22; }
	  if (tmp_pt2->element == "OH") { tmp_str.append("O"); hs += 22; }
	  if (tmp_pt2->element == "HO") { tmp_str.append("O"); hs += 22; }
	  if (tmp_pt2->element == "N") { tmp_str.append("N"); hs += 20; }
	  if (tmp_pt2->element == "NH2") { tmp_str.append("N"); hs += 20; }
	  if (tmp_pt2->element == "H2N") { tmp_str.append("N"); hs += 20; }
	  if (tmp_pt2->element == "NH") { tmp_str.append("N"); hs += 20; }
	  if (tmp_pt2->element == "HN") { tmp_str.append("N"); hs += 20; }
	  if (tmp_pt2->element == "P") { tmp_str.append("P"); hs += 18; }
	  if (tmp_pt2->element == "S") { tmp_str.append("S"); hs += 16; }
	  if (tmp_pt2->element == "Si") { tmp_str.append("Q"); hs += 14; }
	  if (tmp_pt2->element == "B") { tmp_str.append("B"); hs += 12; }
	  if (tmp_pt2->element == "F") { tmp_str.append("F"); hs += 10; }
	  if (tmp_pt2->element == "Cl") { tmp_str.append("X"); hs += 8; }
	  if (tmp_pt2->element == "Br") { tmp_str.append("Y"); hs += 6; }
	  if (tmp_pt2->element == "I") { tmp_str.append("I"); hs += 4; }
	  if (tmp_pt2->ketos == 2) { tmp_str.append("$$"); }
	  if (tmp_pt2->ketos == 1) { tmp_str.append("$"); }
	  tmp_pt2->tmphose = tmp_str;
	  tmp_pt2->hosescore = hs;
	  sphere1.append(tmp_pt2);
	}
      }
      tmp_str = "";
      do {
	hsmax = -1;
	for (tmp_pt3 = sphere1.first(); tmp_pt3 != 0;
	     tmp_pt3 = sphere1.next()) {
	  if (tmp_pt3->hosescore > hsmax) {
	    hsmax = tmp_pt3->hosescore;
	    ts = tmp_pt3->tmphose;
	    tmp_pt2 = tmp_pt3;
	  }
	}
	sphere1.remove(tmp_pt2);
	tmp_str.append(ts);
      } while (sphere1.count() > 0);
      tmp_str.append("(//)");
      hosecodes.append(tmp_str);
      cout << tmp_str << endl;
    }
  }

  if (show_dialog == false) return hosecodes;

  GraphDialog g(r, i18n( "Predicted 13C-NMR" ));
  // print (for now) list of NMR values
  for ( QStringList::Iterator it = hosecodes.begin();
	it != hosecodes.end(); ++it ) {
    //cout << boshcp.predictFull(*it) << endl;
    g.AddPeak(boshcp.predict(*it), boshcp.getMult(*it),
	      boshcp.predictFull(*it));
  }

  QPixmap mol = r->MakeFullPixmap();
  QRect rr1 = BoundingBoxAll();
  rr1.setLeft(rr1.left() - 4);
  rr1.setTop(rr1.top() - 4);
  rr1.setBottom(rr1.bottom() + 4);
  rr1.setRight(rr1.right() + 4);
  QPixmap mol1(rr1.size());
  bitBlt(&mol1, QPoint(0,0), &mol, rr1);

  g.AddPixmap(mol1);
  g.exec();

  return hosecodes;
}
