// prefs.h -- static Preferences class

#include <qfile.h>
#include <qtextstream.h>

class Preferences {
 public:
  Preferences() { Defaults(); }
  void Defaults() {
    fixed_arrow = true;
    arrow_fixedlength = 50.0;
    arrow_fixedangle = 15.0;
    fixed_bond = true;
    bond_fixedlength = 25.0;
    bond_fixedangle = 15.0;
  }
  void setFile(QString fn, bool fb) {
    fileName = fn;
    fallback = fb;
  }
  void setSaveFile(QString fn) {
    saveFileName = fn;
  }
  bool LoadPrefs() {
    QFile fin(fileName);
    
    if (fin.open(IO_ReadOnly) == false)
      return false;

    QTextStream tin(&fin);
    QString line;
    do {
      line = tin.readLine();
      //cout << "PREFS:" << line << endl;
      if (line.upper().contains("FIXED_ARROW") > 0) {
	if (line.upper().contains("TRUE"))
	  fixed_arrow = true;
	else
	  fixed_arrow = false;
      }
      if (line.upper().contains("FIXED_BOND") > 0) {
	if (line.upper().contains("TRUE"))
	  fixed_bond = true;
	else
	  fixed_bond = false;
      }
      if (line.upper().contains("ARROW_LENGTH") > 0) {
	arrow_fixedlength = line.mid(13).toDouble();
      }
      if (line.upper().contains("ARROW_ANGLE") > 0) {
	arrow_fixedangle = line.mid(12).toDouble();
      }
      if (line.upper().contains("BOND_LENGTH") > 0) {
	bond_fixedlength = line.mid(12).toDouble();
      }
      if (line.upper().contains("BOND_ANGLE") > 0) {
	bond_fixedangle = line.mid(11).toDouble();
      }
    } while (!tin.atEnd());

    fin.close();

    return true;
  }
  bool SavePrefs() {
    QFile fout(saveFileName);
    
    if (fout.open(IO_WriteOnly) == false)
      return false;

    QTextStream tout(&fout);
    if (fixed_arrow)
      tout << "FIXED_ARROW true" << endl;
    else
      tout << "FIXED_ARROW false" << endl;
    if (fixed_bond)
      tout << "FIXED_BOND true" << endl;
    else
      tout << "FIXED_BOND false" << endl;
    tout << "ARROW_LENGTH " << arrow_fixedlength << endl;
    tout << "ARROW_ANGLE " << arrow_fixedangle << endl;
    tout << "BOND_LENGTH " << bond_fixedlength << endl;
    tout << "BOND_ANGLE " << bond_fixedangle << endl;

    fout.close();

    return true;
  }
  bool getArrow_fixed() { return fixed_arrow; }
  double getArrow_fixedlength() { return arrow_fixedlength; }
  double getArrow_fixedangle() { return arrow_fixedangle; }
  bool getBond_fixed() { return fixed_bond; }
  double getBond_fixedlength() { return bond_fixedlength; }
  double getBond_fixedangle() { return bond_fixedangle; }
  bool getAntialiased() { return antialiased; }
  void setArrow_fixed(bool n1) { fixed_arrow = n1; }
  void setArrow_fixedlength(double n1) { arrow_fixedlength = n1; }
  void setArrow_fixedangle(double n1) { arrow_fixedangle = n1; }
  void setBond_fixed(bool n1) { fixed_bond = n1; }
  void setBond_fixedlength(double n1) { bond_fixedlength = n1; }
  void setBond_fixedangle(double n1) { bond_fixedangle = n1; }
  void setAntialiased(bool n1) { antialiased = n1; }

 private:
  bool fixed_arrow, fixed_bond;
  bool antialiased;
  bool fallback; // true = file from library directory
  QString fileName;  // prefs file name
  QString saveFileName;  // save file name
  double arrow_fixedlength;
  double arrow_fixedangle;
  double bond_fixedlength;
  double bond_fixedangle;
};
