/* -*- C++ -*-

Copyright (C) 2002 Francesco Bresciani <fbresciani@users.sourceforge.net>
This file is part of Kemistry

Kemistry is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Kemistry is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
NU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Kemistry; see the file COPYING.  If not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include <kaboutdata.h>
#include <kaboutdialog.h>
#include <kaboutapplication.h>
#include <kapp.h>
#include <kbugreport.h>
#include <kdebug.h>
#include <kglobal.h>
#include <klocale.h>
#include <kurlrequester.h>

#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qradiobutton.h>

#include "kembabel.h"

KemBabelWin::KemBabelWin()
{
	
	_widget = new KemBabelWidget(this);

    connect(_widget->_btnClose, SIGNAL(clicked()),
            this, SLOT(close()));
    connect(_widget->_btnAbout, SIGNAL(clicked()), this,
            SLOT(showAboutApplication()));
    connect(_widget->_btnReportBug, SIGNAL(clicked()), this,
            SLOT(slotReportBug()));
	connect(_widget->_btnHelp, SIGNAL(clicked()), this,
			SLOT(appHelpActivated()));
    setCentralWidget(_widget);

	adjustSize();
}



KemBabelWin::~KemBabelWin()
{
}

void KemBabelWin::slotReportBug()
{
  static KBugReport* bugReport = 0L;
  
  if (bugReport)
    bugReport->show();
  else
  {
    bugReport= new KBugReport(this);
    bugReport->show();
  }
}

void KemBabelWin::showAboutApplication()
{
    static KAboutApplication* about = 0L;
    if (about)
        about->show();
    else
    {
        about = new KAboutApplication(this);
		about->addTextPage( i18n( "About Open Babel" ),
				i18n( "This application is linked to the Open Babel library."
					"<br><br>Open Babel is the open source version of Babel, "
					"a chemistry file translation program.<br><br>"
					"For more informations about Open Babel go to the "
					"<a href=\"http://openbabel.sourceforge.net\">Open Babel "
					" homepage</a>" ), true );
        about->show();
    }
}

void KemBabelWidget::slotAddHpHChecked(  bool on )
{
	if ( on )
	// check the Add Hydrogens check box and disable it
	{
		_checkAddH->setChecked( true );
		_checkAddH->setEnabled( false );
	}
	else
	// enable the Add Hydrogens check box
		_checkAddH->setEnabled( true );
}

void KemBabelWidget::slotCMLNamespaceChecked( bool on )
{
	if ( on )
	//check the Output Namespace check box and disable it
	{
		_checkNamespace->setChecked( true );
		_checkNamespace->setEnabled( false );
	}
	else
	// enable the Output Namespace check box 
		_checkNamespace->setEnabled( true );
}

KemBabelWidget::KemBabelWidget( QWidget * parent ) : 
KemBabelWidgetBase( parent, "KEMBABELWIDGET" )
{
	_config = kapp->config();
	loadConfig();

	_KURLReqOutput->setMode( KFile::File );
	
	fillCombos();
}


void KemBabelWidget::loadConfig()
{
	_config->setGroup( "UI" );
	//first page (Transformations)
	_checkDeleteH->setChecked( _config->readBoolEntry( "DeleteH" ) );
	_checkAddH->setChecked( _config->readBoolEntry( "AddH" ) );
	_checkAddHpH->setChecked( _config->readBoolEntry( "AddHpH" ) );
	//second page (CML)
	_radioCMLV1->setChecked( _config->readBoolEntry( "CMLV1", true ) );
	_checkNamespace->setChecked( _config->readBoolEntry( "Namespace" ) );
	_checkCMLNamespace->setChecked( _config->readBoolEntry( "CMLNamespace" ) );
	_checkDOCTYPE->setChecked( _config->readBoolEntry( "DOCTYPE" ) );
	_checkArray->setChecked( _config->readBoolEntry( "Array" ) );
	_checkPretty->setChecked( _config->readBoolEntry( "PrettyPrint" ) );
}

void KemBabelWidget::saveConfig()
{
	_config->setGroup( "UI" );
	_config->writeEntry( "DeleteH", _checkDeleteH->isChecked() );
	_config->writeEntry( "AddH", _checkAddH->isChecked() );
	_config->writeEntry( "AddHpH", _checkAddHpH->isChecked() );
	_config->writeEntry( "CMLV1", _radioCMLV1->isChecked() );
	_config->writeEntry( "Namespace", _checkNamespace->isChecked() );
	_config->writeEntry( "CMLNamespace", _checkCMLNamespace->isChecked() );
	_config->writeEntry( "DOCTYPE", _checkDOCTYPE->isChecked() );
	_config->writeEntry( "Array", _checkArray->isChecked() );
	_config->writeEntry( "PrettyPrint", _checkPretty->isChecked() );
	
}

KemBabelWidget::~KemBabelWidget()
{
	saveConfig();
}

#include "kembabel.moc"
